@@ -1,38 +1,26 @@
 import os
 from datetime import datetime
+from functools import wraps
 from random import random
-from .monitor import Monitor, MonitorNotFound, MonitorNotCreated, MonitorNotUpdated
+from .monitor import Monitor, MonitorNotFound, MonitorNotCreated, MonitorNotUpdated, InvalidPingUrl, InvalidMonitorParams
 import logging
 
 logging.basicConfig()
 logger = logging.getLogger(__name__)
 
-def ping(name, schedule=None, rules=[], notifications={}, timezone=None, api_key=None, type=None):
+def ping(key, id=None, schedule=None, api_key=None):
     def wrapper(func):
-
+        @wraps(func)
         def wrapped(*args, **kwargs):
-            create_args = {
-                'name':name,
-                'schedule':schedule,
-                'rules':rules,
-                'notifications':notifications,
-                'timezone':timezone,}
-
-            if api_key:
-                create_args['api_key'] = api_key
-
-            if type:
-                create_args['type'] = type
+            start = datetime.now().timestamp()
 
             try:
-                monitor = Monitor.get_or_create(**create_args)
-            except Exception as e:
-                logger.debug(str(e))
+                monitor = Monitor(id, key=key)
+            except Exception:
                 return func(*args, **kwargs)
 
-            start = datetime.now().timestamp()
+            # use start as the series param to match run/completes correctly
             monitor.run(stamp=start, series=start)
-
             try:
                 out = func(*args, **kwargs)
             except Exception as e:
