@@ -12,30 +12,34 @@ def main():
                         default=os.getenv('CRONITOR_PING_API_KEY', os.getenv('CRONITOR_AUTH_KEY')),
                         help='Auth Key from Account page')
     parser.add_argument('--id', '-i', type=str,
-                        default=os.getenv('CRONITOR_CODE', os.getenv('CRONITOR_ID')),
-                        help='Id for Monitor to take action upon')
-
+                        default=os.getenv('CRONITOR_ID', os.getenv('CRONITOR_CODE')),
+                        help='Monitor Id to take action upon')
     # alias for id. deprecated.
     parser.add_argument('--code', '-c', type=str,
                         default=os.getenv('CRONITOR_CODE'),
-                        help='Code for Monitor to take action upon')
+                        help='DEPRECATED: Code for Monitor to take action upon. Alias of Id.')
     parser.add_argument('--msg', '-m', type=str, default='',
                         help='Optional message to send with ping/fail')
 
     group = parser.add_mutually_exclusive_group(required=True)
+
     group.add_argument('--run', '-r', action='store_true',
-                       help='Call ping on given Code')
+                       help='Send a run event')
     group.add_argument('--complete', '-C', action='store_true',
-                       help='Call complete on given Code')
+                       help='Send a complete event')
+    group.add_argument('--tick', '-t', action='store_true',
+                       help='Send an tick event')
     group.add_argument('--fail', '-f', action='store_true',
-                       help='Call fail on given Code')
-    group.add_argument('--pause', '-P', type=str, default=24,
-                       help='Call pause on given Code')
+                       help='Send a fail event')
+    group.add_argument('--ok', '-o', action='store_true',
+                       help='Send an ok event')
+    group.add_argument('--pause', '-p', type=str, default=24,
+                       help='Pause a monitor')
 
     args = parser.parse_args()
 
-    if args.code and args.id is None:
-        print('A code/id must be supplied or CRONITOR_CODE ENV var used')
+    if args.id is None and args.code is None:
+        print('A monitorId must be supplied using the --id flag or setting the CRONITOR_ID enviromenment variable.')
         parser.print_help()
         sys.exit(1)
 
@@ -43,14 +47,18 @@ def main():
 
     if args.run:
         ret = monitor.run(message=args.msg)
-    elif args.fail:
-        ret = monitor.failed(message=args.msg)
     elif args.complete:
         ret = monitor.complete(message=args.msg)
+    elif args.tick:
+        ret = monitor.tick(message=args.msg)
+    elif args.fail:
+        ret = monitor.fail(message=args.msg)
+    elif args.ok:
+        ret = monitor.ok(message=args.msg)
     elif args.pause:
         ret = monitor.pause(args.pause)
 
-    return ret.raise_for_status()
+    return ret
 
 
 if __name__ == '__main__':
